@@ -29,6 +29,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_fileops.h"
+#include "BLI_listbase.h"
 #include "BLI_path_utils.hh"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
@@ -36,6 +37,7 @@
 #include "DEG_depsgraph.hh"
 
 #include "BKE_idtype.hh"
+#include "BKE_library.hh"
 #include "BKE_main.hh"
 #include "BKE_node.hh"
 #include "BKE_report.hh"
@@ -48,7 +50,7 @@
 #  include "BLI_strict_flags.h" /* IWYU pragma: keep. Keep last. */
 #endif
 
-static CLG_LogRef LOG = {"bke.bpath"};
+static CLG_LogRef LOG = {"lib.bpath"};
 
 /* -------------------------------------------------------------------- */
 /** \name Generic Utilities
@@ -231,7 +233,7 @@ static bool check_missing_files_foreach_path_cb(BPathForeachPathData *bpath_data
                     "Path '%s' not found, from linked data-block '%s' (from library '%s')",
                     path_src,
                     owner_id->name,
-                    owner_id->lib->runtime.filepath_abs);
+                    owner_id->lib->runtime->filepath_abs);
       }
       else {
         BKE_reportf(reports,
@@ -303,6 +305,7 @@ static bool missing_files_find__recursive(const char *search_directory,
   int64_t size;
   bool found = false;
 
+  BLI_assert(!BLI_path_is_rel(search_directory));
   dir = opendir(search_directory);
 
   if (dir == nullptr) {
@@ -688,7 +691,7 @@ static bool bpath_list_restore(BPathForeachPathData *bpath_data,
 
 void *BKE_bpath_list_backup(Main *bmain, const eBPathForeachFlag flag)
 {
-  ListBase *path_list = static_cast<ListBase *>(MEM_callocN(sizeof(ListBase), __func__));
+  ListBase *path_list = MEM_callocN<ListBase>(__func__);
 
   BPathForeachPathData path_data{};
   path_data.bmain = bmain;
