@@ -2,6 +2,10 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
+/** \file
+ * \ingroup bli
+ */
+
 #include "BLI_array_utils.hh"
 #include "BLI_offset_indices.hh"
 #include "BLI_task.hh"
@@ -25,7 +29,7 @@ OffsetIndices<int> accumulate_counts_to_offsets(MutableSpan<int> counts_to_offse
   }
   counts_to_offsets.last() = offset;
 
-  BLI_assert_msg(offset == offset_i64, "Integer overflow occured");
+  BLI_assert_msg(offset == offset_i64, "Integer overflow occurred");
   UNUSED_VARS_NDEBUG(offset_i64);
 
   return OffsetIndices<int>(counts_to_offsets);
