@@ -6,23 +6,21 @@
  * \ingroup edgpencil
  */
 
+#include <algorithm>
 #include <cmath>
-#include <cstddef>
-#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_blenlib.h"
+#include "BLI_listbase.h"
+#include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_gpencil_legacy_types.h"
 #include "DNA_scene_types.h"
 
-#include "BKE_fcurve.h"
 #include "BKE_gpencil_legacy.h"
-#include "BKE_report.h"
 
 #include "ED_anim_api.hh"
 #include "ED_gpencil_legacy.hh"
@@ -78,7 +76,7 @@ void ED_gpencil_layer_make_cfra_list(bGPDlayer *gpl, ListBase *elems, bool onlys
   /* loop through gp-frames, adding */
   LISTBASE_FOREACH (bGPDframe *, gpf, &gpl->frames) {
     if ((onlysel == 0) || (gpf->flag & GP_FRAME_SELECT)) {
-      ce = static_cast<CfraElem *>(MEM_callocN(sizeof(CfraElem), "CfraElem"));
+      ce = MEM_callocN<CfraElem>("CfraElem");
 
       ce->cfra = float(gpf->framenum);
       ce->sel = (gpf->flag & GP_FRAME_SELECT) ? 1 : 0;
@@ -223,7 +221,6 @@ void ED_gpencil_set_active_channel(bGPdata *gpd, bGPDlayer *gpl)
   /* Update other layer status. */
   if (BKE_gpencil_layer_active_get(gpd) != gpl) {
     BKE_gpencil_layer_active_set(gpd, gpl);
-    BKE_gpencil_layer_autolock_set(gpd, false);
     WM_main_add_notifier(NC_GPENCIL | ND_DATA | NA_EDITED, nullptr);
   }
 }
@@ -332,7 +329,7 @@ bool ED_gpencil_anim_copybuf_copy(bAnimContext *ac)
 
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
     /* This function only deals with grease pencil layer frames.
-     * This check is needed in the case of a call from the main dopesheet. */
+     * This check is needed in the case of a call from the main dope-sheet. */
     if (ale->type != ANIMTYPE_GPLAYER) {
       continue;
     }
@@ -349,19 +346,14 @@ bool ED_gpencil_anim_copybuf_copy(bAnimContext *ac)
         BLI_addtail(&copied_frames, new_frame);
 
         /* extend extents for keyframes encountered */
-        if (gpf->framenum < gpencil_anim_copy_firstframe) {
-          gpencil_anim_copy_firstframe = gpf->framenum;
-        }
-        if (gpf->framenum > gpencil_anim_copy_lastframe) {
-          gpencil_anim_copy_lastframe = gpf->framenum;
-        }
+        gpencil_anim_copy_firstframe = std::min(gpf->framenum, gpencil_anim_copy_firstframe);
+        gpencil_anim_copy_lastframe = std::max(gpf->framenum, gpencil_anim_copy_lastframe);
       }
     }
 
     /* create a new layer in buffer if there were keyframes here */
     if (BLI_listbase_is_empty(&copied_frames) == false) {
-      bGPDlayer *new_layer = static_cast<bGPDlayer *>(
-          MEM_callocN(sizeof(bGPDlayer), "GPCopyPasteLayer"));
+      bGPDlayer *new_layer = MEM_callocN<bGPDlayer>("GPCopyPasteLayer");
       BLI_addtail(&gpencil_anim_copybuf, new_layer);
 
       /* move over copied frames */
@@ -369,7 +361,7 @@ bool ED_gpencil_anim_copybuf_copy(bAnimContext *ac)
       BLI_assert(copied_frames.first == nullptr);
 
       /* make a copy of the layer's name - for name-based matching later... */
-      STRNCPY(new_layer->info, gpl->info);
+      STRNCPY_UTF8(new_layer->info, gpl->info);
     }
   }
 
@@ -427,7 +419,7 @@ bool ED_gpencil_anim_copybuf_paste(bAnimContext *ac, const short offset_mode)
 
   /* from selected channels */
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
-    /* only deal with GPlayers (case of calls from general dopesheet) */
+    /* Only deal with GPlayers (case of calls from general dope-sheet). */
     if (ale->type != ANIMTYPE_GPLAYER) {
       continue;
     }
@@ -468,7 +460,7 @@ bool ED_gpencil_anim_copybuf_paste(bAnimContext *ac, const short offset_mode)
          *   if it works, it will show up.
          */
         LISTBASE_FOREACH (bGPDstroke *, gps, &gpfs->strokes) {
-          /* make a copy of stroke, then of its points array */
+          /* Make a copy of stroke, then a copy of its points array. */
           bGPDstroke *gpsn = BKE_gpencil_stroke_duplicate(gps, true, true);
 
           /* append stroke to frame */
@@ -509,7 +501,7 @@ static bool gpencil_frame_snap_nearest(bGPDframe * /*gpf*/, Scene * /*scene*/)
 
 static bool gpencil_frame_snap_nearestsec(bGPDframe *gpf, Scene *scene)
 {
-  float secf = float(FPS);
+  float secf = float(scene->frames_per_second());
   if (gpf->flag & GP_FRAME_SELECT) {
     gpf->framenum = int(floorf(gpf->framenum / secf + 0.5f) * secf);
   }
@@ -519,7 +511,7 @@ static bool gpencil_frame_snap_nearestsec(bGPDframe *gpf, Scene *scene)
 static bool gpencil_frame_snap_cframe(bGPDframe *gpf, Scene *scene)
 {
   if (gpf->flag & GP_FRAME_SELECT) {
-    gpf->framenum = int(scene->r.cfra);
+    gpf->framenum = scene->r.cfra;
   }
   return false;
 }
@@ -527,8 +519,7 @@ static bool gpencil_frame_snap_cframe(bGPDframe *gpf, Scene *scene)
 static bool gpencil_frame_snap_nearmarker(bGPDframe *gpf, Scene *scene)
 {
   if (gpf->flag & GP_FRAME_SELECT) {
-    gpf->framenum = (int)ED_markers_find_nearest_marker_time(&scene->markers,
-                                                             float(gpf->framenum));
+    gpf->framenum = ED_markers_find_nearest_marker_time(&scene->markers, float(gpf->framenum));
   }
   return false;
 }
