@@ -25,8 +25,6 @@
 #include "RNA_define.hh"
 #include "RNA_path.hh"
 
-#include "UI_interface.hh"
-
 #include "WM_api.hh"
 #include "WM_types.hh"
 
@@ -127,7 +125,7 @@ static void driverdropper_cancel(bContext *C, wmOperator *op)
 }
 
 /* main modal status check */
-static int driverdropper_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus driverdropper_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   DriverDropper *ddr = static_cast<DriverDropper *>(op->customdata);
 
@@ -152,7 +150,9 @@ static int driverdropper_modal(bContext *C, wmOperator *op, const wmEvent *event
 }
 
 /* Modal Operator init */
-static int driverdropper_invoke(bContext *C, wmOperator *op, const wmEvent * /*event*/)
+static wmOperatorStatus driverdropper_invoke(bContext *C,
+                                             wmOperator *op,
+                                             const wmEvent * /*event*/)
 {
   /* init */
   if (driverdropper_init(C, op)) {
@@ -170,7 +170,7 @@ static int driverdropper_invoke(bContext *C, wmOperator *op, const wmEvent * /*e
 }
 
 /* Repeat operator */
-static int driverdropper_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus driverdropper_exec(bContext *C, wmOperator *op)
 {
   /* init */
   if (driverdropper_init(C, op)) {
@@ -197,7 +197,7 @@ void UI_OT_eyedropper_driver(wmOperatorType *ot)
   ot->idname = "UI_OT_eyedropper_driver";
   ot->description = "Pick a property to use as a driver target";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = driverdropper_invoke;
   ot->modal = driverdropper_modal;
   ot->cancel = driverdropper_cancel;
