@@ -42,6 +42,7 @@
 #include "RNA_prototypes.hh"
 
 #include "UI_interface.hh"
+#include "UI_interface_layout.hh"
 #include "UI_resources.hh"
 
 #include <climits>
@@ -766,7 +767,7 @@ static void grease_pencil_interpolate_update(bContext &C, const wmOperator &op)
       geometry::smooth_curve_attribute(
           interpolated_curves.curves_range(),
           interpolated_curves.points_by_curve(),
-          VArray<bool>::ForSingle(true, interpolated_curves.points_num()),
+          VArray<bool>::from_single(true, interpolated_curves.points_num()),
           interpolated_curves.cyclic(),
           opdata.smooth_steps,
           opdata.smooth_factor,
@@ -1308,7 +1309,7 @@ static wmOperatorStatus grease_pencil_interpolate_sequence_exec(bContext *C, wmO
         geometry::smooth_curve_attribute(
             interpolated_curves.curves_range(),
             interpolated_curves.points_by_curve(),
-            VArray<bool>::ForSingle(true, interpolated_curves.points_num()),
+            VArray<bool>::from_single(true, interpolated_curves.points_num()),
             interpolated_curves.cyclic(),
             opdata.smooth_steps,
             opdata.smooth_factor,
@@ -1340,8 +1341,8 @@ static void grease_pencil_interpolate_sequence_ui(bContext *C, wmOperator *op)
 
   const InterpolationType type = InterpolationType(RNA_enum_get(op->ptr, "type"));
 
-  uiLayoutSetPropSep(layout, true);
-  uiLayoutSetPropDecorate(layout, false);
+  layout->use_property_split_set(true);
+  layout->use_property_decorate_set(false);
   row = &layout->row(true);
   row->prop(op->ptr, "step", UI_ITEM_NONE, std::nullopt, ICON_NONE);
 
@@ -1376,7 +1377,7 @@ static void grease_pencil_interpolate_sequence_ui(bContext *C, wmOperator *op)
     PointerRNA gpsettings_ptr = RNA_pointer_create_discrete(
         &scene->id, &RNA_GPencilInterpolateSettings, &ts->gp_interpolate);
     uiTemplateCurveMapping(
-        layout, &gpsettings_ptr, "interpolation_curve", 0, false, true, true, false);
+        layout, &gpsettings_ptr, "interpolation_curve", 0, false, true, true, false, false);
   }
   else if (type != InterpolationType::Linear) {
     row = &layout->row(false);
