@@ -24,6 +24,7 @@
 
 #include "BLI_fileops.h"
 #include "BLI_listbase.h"
+#include "BLI_mutex.hh"
 #include "BLI_rect.h"
 #include "BLI_task.h"
 #include "BLI_utildefines.h"
@@ -104,7 +105,8 @@ bool ED_space_clip_maskedit_visible_splines_poll(bContext *C)
   }
 
   const SpaceClip *space_clip = CTX_wm_space_clip(C);
-  return space_clip->mask_info.draw_flag & MASK_DRAWFLAG_SPLINE;
+  return space_clip->overlay.flag & SC_SHOW_OVERLAYS &&
+         space_clip->mask_info.draw_flag & MASK_DRAWFLAG_SPLINE;
 }
 
 bool ED_space_clip_maskedit_mask_poll(bContext *C)
@@ -129,7 +131,8 @@ bool ED_space_clip_maskedit_mask_visible_splines_poll(bContext *C)
   }
 
   const SpaceClip *space_clip = CTX_wm_space_clip(C);
-  return space_clip->mask_info.draw_flag & MASK_DRAWFLAG_SPLINE;
+  return space_clip->overlay.flag & SC_SHOW_OVERLAYS &&
+         space_clip->mask_info.draw_flag & MASK_DRAWFLAG_SPLINE;
 }
 
 /** \} */
@@ -681,7 +684,7 @@ struct PrefetchQueue {
    */
   bool forward;
 
-  SpinLock spin;
+  blender::Mutex mutex;
 
   bool *stop;
   bool *do_update;
@@ -717,7 +720,7 @@ static uchar *prefetch_read_file_to_memory(
     return nullptr;
   }
 
-  uchar *mem = MEM_cnew_array<uchar>(size, "movieclip prefetch memory file");
+  uchar *mem = MEM_calloc_arrayN<uchar>(size, "movieclip prefetch memory file");
   if (mem == nullptr) {
     close(file);
     return nullptr;
@@ -780,7 +783,7 @@ static uchar *prefetch_thread_next_frame(PrefetchQueue *queue,
 {
   uchar *mem = nullptr;
 
-  BLI_spin_lock(&queue->spin);
+  std::lock_guard lock(queue->mutex);
   if (!*queue->stop && !check_prefetch_break() &&
       IN_RANGE_INCL(queue->current_frame, queue->start_frame, queue->end_frame))
   {
@@ -831,7 +834,6 @@ static uchar *prefetch_thread_next_frame(PrefetchQueue *queue,
       *queue->progress = float(frames_processed) / (queue->end_frame - queue->start_frame);
     }
   }
-  BLI_spin_unlock(&queue->spin);
 
   return mem;
 }
@@ -847,7 +849,7 @@ static void prefetch_task_func(TaskPool *__restrict pool, void *task_data)
   while ((mem = prefetch_thread_next_frame(queue, clip, &size, &current_frame))) {
     ImBuf *ibuf;
     MovieClipUser user = *DNA_struct_default_get(MovieClipUser);
-    int flag = IB_rect | IB_multilayer | IB_alphamode_detect | IB_metadata;
+    int flag = IB_byte_data | IB_multilayer | IB_alphamode_detect | IB_metadata;
     int result;
     char *colorspace_name = nullptr;
     const bool use_proxy = (clip->flag & MCLIP_USE_PROXY) &&
@@ -862,7 +864,7 @@ static void prefetch_task_func(TaskPool *__restrict pool, void *task_data)
       colorspace_name = clip->colorspace_settings.name;
     }
 
-    ibuf = IMB_ibImageFromMemory(mem, size, flag, colorspace_name, "prefetch frame");
+    ibuf = IMB_load_image_from_memory(mem, size, flag, "prefetch frame", nullptr, colorspace_name);
     if (ibuf == nullptr) {
       continue;
     }
@@ -896,8 +898,6 @@ static void start_prefetch_threads(MovieClip *clip,
 
   /* initialize queue */
   PrefetchQueue queue;
-  BLI_spin_init(&queue.spin);
-
   queue.current_frame = current_frame;
   queue.initial_frame = current_frame;
   queue.start_frame = start_frame;
@@ -916,8 +916,6 @@ static void start_prefetch_threads(MovieClip *clip,
   }
   BLI_task_pool_work_and_wait(task_pool);
   BLI_task_pool_free(task_pool);
-
-  BLI_spin_end(&queue.spin);
 }
 
 /* NOTE: Reading happens from `clip_local` into `clip->cache`. */
@@ -1121,12 +1119,12 @@ void clip_start_prefetch_job(const bContext *C)
   wm_job = WM_jobs_get(CTX_wm_manager(C),
                        CTX_wm_window(C),
                        CTX_data_scene(C),
-                       "Prefetching",
+                       "Prefetching...",
                        WM_JOB_PROGRESS,
                        WM_JOB_TYPE_CLIP_PREFETCH);
 
   /* create new job */
-  pj = MEM_cnew<PrefetchJob>("prefetch job");
+  pj = MEM_callocN<PrefetchJob>("prefetch job");
   pj->clip = ED_space_clip_get_clip(sc);
   pj->start_frame = prefetch_get_start_frame(C);
   pj->current_frame = sc->user.framenr;
