@@ -259,7 +259,7 @@ static void do_outliner_object_select_recursive(const Scene *scene,
 static void do_outliner_bone_select_recursive(bArmature *arm, Bone *bone_parent, bool select)
 {
   LISTBASE_FOREACH (Bone *, bone, &bone_parent->childbase) {
-    if (select && PBONE_SELECTABLE(arm, bone)) {
+    if (select && blender::animrig::bone_is_selectable(arm, bone)) {
       bone->flag |= BONE_SELECTED;
     }
     else {
@@ -564,7 +564,7 @@ static void tree_element_posechannel_activate(bContext *C,
       }
 
       LISTBASE_FOREACH (bPoseChannel *, pchannel, &ob_iter->pose->chanbase) {
-        pchannel->bone->flag &= ~(BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL);
+        pchannel->flag &= ~POSE_SELECTED;
       }
 
       if (ob != ob_iter) {
@@ -573,19 +573,19 @@ static void tree_element_posechannel_activate(bContext *C,
     }
   }
 
-  if ((set == OL_SETSEL_EXTEND) && (pchan->bone->flag & BONE_SELECTED)) {
-    pchan->bone->flag &= ~BONE_SELECTED;
+  if ((set == OL_SETSEL_EXTEND) && (pchan->flag & POSE_SELECTED)) {
+    pchan->flag &= ~POSE_SELECTED;
   }
   else {
-    if (blender::animrig::bone_is_visible(arm, pchan->bone)) {
-      pchan->bone->flag |= BONE_SELECTED;
+    if (blender::animrig::bone_is_visible(arm, pchan)) {
+      pchan->flag |= POSE_SELECTED;
     }
     arm->act_bone = pchan->bone;
   }
 
   if (recursive) {
     /* Recursive select/deselect */
-    do_outliner_bone_select_recursive(arm, pchan->bone, (pchan->bone->flag & BONE_SELECTED) != 0);
+    do_outliner_bone_select_recursive(arm, pchan->bone, (pchan->flag & POSE_SELECTED) != 0);
   }
 
   WM_event_add_notifier(C, NC_OBJECT | ND_BONE_ACTIVE, ob);
@@ -730,40 +730,48 @@ static void tree_element_constraint_activate(bContext *C,
 }
 
 static void tree_element_strip_activate(bContext *C,
-                                        Scene *scene,
+                                        WorkSpace *workspace,
                                         TreeElement *te,
                                         const eOLSetState set)
 {
+  Scene *sequencer_scene = workspace->sequencer_scene;
+  if (!sequencer_scene) {
+    return;
+  }
   const TreeElementStrip *te_strip = tree_element_cast<TreeElementStrip>(te);
   Strip *strip = &te_strip->get_strip();
-  Editing *ed = seq::editing_get(scene);
+  Editing *ed = seq::editing_get(sequencer_scene);
 
-  if (BLI_findindex(ed->seqbasep, strip) != -1) {
+  if (BLI_findindex(ed->current_strips(), strip) != -1) {
     if (set == OL_SETSEL_EXTEND) {
-      seq::select_active_set(scene, nullptr);
+      seq::select_active_set(sequencer_scene, nullptr);
     }
-    vse::deselect_all_strips(scene);
+    vse::deselect_all_strips(sequencer_scene);
 
     if ((set == OL_SETSEL_EXTEND) && strip->flag & SELECT) {
       strip->flag &= ~SELECT;
     }
     else {
       strip->flag |= SELECT;
-      seq::select_active_set(scene, strip);
+      seq::select_active_set(sequencer_scene, strip);
     }
   }
 
-  WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER | NA_SELECTED, scene);
+  WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER | NA_SELECTED, sequencer_scene);
 }
 
-static void tree_element_strip_dup_activate(Scene *scene, TreeElement * /*te*/)
+static void tree_element_strip_dup_activate(WorkSpace *workspace, TreeElement * /*te*/)
 {
-  Editing *ed = seq::editing_get(scene);
+  Scene *sequencer_scene = workspace->sequencer_scene;
+  if (!sequencer_scene) {
+    return;
+  }
+  Editing *ed = seq::editing_get(sequencer_scene);
 
 #if 0
   select_single_seq(strip, 1);
 #endif
-  Strip *p = static_cast<Strip *>(ed->seqbasep->first);
+  Strip *p = static_cast<Strip *>(ed->current_strips()->first);
   while (p) {
     if ((!p->data) || (!p->data->stripdata) || (p->data->stripdata->filename[0] == '\0')) {
       p = p->next;
@@ -882,10 +890,10 @@ void tree_element_type_active_set(bContext *C,
       tree_element_bonecollection_activate(C, te, tselem);
       break;
     case TSE_STRIP:
-      tree_element_strip_activate(C, tvc.scene, te, set);
+      tree_element_strip_activate(C, tvc.workspace, te, set);
       break;
     case TSE_STRIP_DUP:
-      tree_element_strip_dup_activate(tvc.scene, te);
+      tree_element_strip_dup_activate(tvc.workspace, te);
       break;
     case TSE_GP_LAYER:
       tree_element_gplayer_activate(C, te, tselem);
@@ -984,7 +992,7 @@ static eOLDrawState tree_element_posechannel_state_get(const Object *ob_pose,
   const Object *ob = (const Object *)tselem->id;
   const bPoseChannel *pchan = static_cast<bPoseChannel *>(te->directdata);
   if (ob == ob_pose && ob->pose) {
-    if (pchan->bone->flag & BONE_SELECTED) {
+    if (pchan->flag & POSE_SELECTED) {
       return OL_DRAWSEL_NORMAL;
     }
   }
@@ -1014,11 +1022,15 @@ static eOLDrawState tree_element_bone_collection_state_get(const TreeElement *te
   return OL_DRAWSEL_NONE;
 }
 
-static eOLDrawState tree_element_strip_state_get(const Scene *scene, const TreeElement *te)
+static eOLDrawState tree_element_strip_state_get(const WorkSpace *workspace, const TreeElement *te)
 {
+  const Scene *sequencer_scene = workspace->sequencer_scene;
+  if (!sequencer_scene) {
+    return OL_DRAWSEL_NONE;
+  }
   const TreeElementStrip *te_strip = tree_element_cast<TreeElementStrip>(te);
   const Strip *strip = &te_strip->get_strip();
-  const Editing *ed = scene->ed;
+  const Editing *ed = seq::editing_get(sequencer_scene);
 
   if (ed && ed->act_strip == strip && strip->flag & SELECT) {
     return OL_DRAWSEL_NORMAL;
@@ -1189,7 +1201,7 @@ eOLDrawState tree_element_type_active_state_get(const TreeViewContext &tvc,
     case TSE_R_LAYER:
       return tree_element_viewlayer_state_get(tvc.view_layer, te);
     case TSE_STRIP:
-      return tree_element_strip_state_get(tvc.scene, te);
+      return tree_element_strip_state_get(tvc.workspace, te);
     case TSE_STRIP_DUP:
       return tree_element_strip_dup_state_get(te);
     case TSE_GP_LAYER:
