@@ -10,10 +10,6 @@
 
 #include "../BPy_UnaryFunction1DVoid.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 extern PyTypeObject ChainingTimeStampF1D_Type;
@@ -27,7 +23,3 @@ typedef struct {
 } BPy_ChainingTimeStampF1D;
 
 ///////////////////////////////////////////////////////////////////////////////////////////
-
-#ifdef __cplusplus
-}
-#endif
