@@ -6,6 +6,8 @@
 #include "BKE_attribute_filters.hh"
 #include "BKE_attribute_math.hh"
 #include "BKE_curves.hh"
+#include "BKE_curves_utils.hh"
+#include "BKE_deform.hh"
 #include "BKE_geometry_set.hh"
 #include "BKE_instances.hh"
 #include "BKE_mesh.hh"
@@ -53,7 +55,7 @@ static void reorder_attributes_group_to_group(const bke::AttributeAccessor src_a
     if (iter.domain != domain) {
       return;
     }
-    if (iter.data_type == CD_PROP_STRING) {
+    if (iter.data_type == bke::AttrType::String) {
       return;
     }
     if (attribute_filter.allow_skip(iter.name)) {
@@ -290,6 +292,12 @@ static void copy_and_reorder_curves(const bke::CurvesGeometry &src_curves,
                                     attribute_filter,
                                     dst_curves.attributes_for_write());
   dst_curves.tag_topology_changed();
+  if (src_curves.nurbs_has_custom_knots()) {
+    dst_curves.nurbs_custom_knots_update_size();
+    IndexMaskMemory memory;
+    bke::curves::nurbs::gather_custom_knots(
+        src_curves, IndexMask::from_indices(old_by_new_map, memory), 0, dst_curves);
+  }
 }
 
 static void copy_and_reorder_instaces(const bke::Instances &src_instances,
@@ -343,6 +351,7 @@ bke::CurvesGeometry reorder_curves_geometry(const bke::CurvesGeometry &src_curve
 {
   bke::CurvesGeometry dst_curves = bke::curves_new_no_attributes(src_curves.points_num(),
                                                                  src_curves.curves_num());
+  BKE_defgroup_copy_list(&dst_curves.vertex_group_names, &src_curves.vertex_group_names);
   copy_and_reorder_curves(src_curves, old_by_new_map, attribute_filter, dst_curves);
   return dst_curves;
 }
@@ -384,9 +393,9 @@ bke::GeometryComponentPtr reordered_component(const bke::GeometryComponent &src_
   if (const bke::PointCloudComponent *src_points_component =
           dynamic_cast<const bke::PointCloudComponent *>(&src_component))
   {
-    PointCloud *result_point_cloud = reorder_points(
+    PointCloud *result_pointcloud = reorder_points(
         *src_points_component->get(), old_by_new_map, attribute_filter);
-    return bke::GeometryComponentPtr(new bke::PointCloudComponent(result_point_cloud));
+    return bke::GeometryComponentPtr(new bke::PointCloudComponent(result_pointcloud));
   }
   if (const bke::CurveComponent *src_curves_component = dynamic_cast<const bke::CurveComponent *>(
           &src_component))
