@@ -20,6 +20,20 @@
 
 #include "WM_api.hh"
 
+const EnumPropertyItem rna_enum_stroke_depth_order_items[] = {
+    {0,
+     "2D",
+     0,
+     "2D Layers",
+     "Display strokes using Grease Pencil layer order and stroke order to define depth"},
+    {GREASE_PENCIL_STROKE_ORDER_3D,
+     "3D",
+     0,
+     "3D Location",
+     "Display strokes using real 3D position in 3D space"},
+    {0, nullptr, 0, nullptr, nullptr},
+};
+
 #ifdef RNA_RUNTIME
 
 #  include <fmt/format.h>
@@ -90,6 +104,7 @@ static void rna_GreasePencilDrawing_curve_offset_data_begin(CollectionPropertyIt
 {
   GreasePencilDrawing *drawing = static_cast<GreasePencilDrawing *>(ptr->data);
   rna_iterator_array_begin(iter,
+                           ptr,
                            drawing->geometry.wrap().offsets_for_write().data(),
                            sizeof(int),
                            drawing->geometry.curve_num + 1,
@@ -105,9 +120,8 @@ static bool rna_GreasePencilDrawing_curve_offset_data_lookup_int(PointerRNA *ptr
   if (index < 0 || index >= drawing->geometry.curve_num + 1) {
     return false;
   }
-  r_ptr->owner_id = ptr->owner_id;
-  r_ptr->type = &RNA_IntAttributeValue;
-  r_ptr->data = &drawing->geometry.wrap().offsets_for_write()[index];
+  rna_pointer_create_with_ancestors(
+      *ptr, &RNA_IntAttributeValue, &drawing->geometry.wrap().offsets_for_write()[index], *r_ptr);
   return true;
 }
 
@@ -117,8 +131,13 @@ static void rna_GreasePencilLayer_frames_begin(CollectionPropertyIterator *iter,
   Layer &layer = static_cast<GreasePencilLayer *>(ptr->data)->wrap();
   blender::Span<FramesMapKeyT> sorted_keys = layer.sorted_keys();
 
-  rna_iterator_array_begin(
-      iter, (void *)sorted_keys.data(), sizeof(FramesMapKeyT), sorted_keys.size(), false, nullptr);
+  rna_iterator_array_begin(iter,
+                           ptr,
+                           (void *)sorted_keys.data(),
+                           sizeof(FramesMapKeyT),
+                           sorted_keys.size(),
+                           false,
+                           nullptr);
 }
 
 static PointerRNA rna_GreasePencilLayer_frames_get(CollectionPropertyIterator *iter)
@@ -127,9 +146,10 @@ static PointerRNA rna_GreasePencilLayer_frames_get(CollectionPropertyIterator *i
   const FramesMapKeyT frame_key = *static_cast<FramesMapKeyT *>(rna_iterator_array_get(iter));
   const Layer &layer = static_cast<GreasePencilLayer *>(iter->parent.data)->wrap();
   const GreasePencilFrame *frame = layer.frames().lookup_ptr(frame_key);
-  return rna_pointer_inherit_refine(&iter->parent,
-                                    &RNA_GreasePencilFrame,
-                                    static_cast<void *>(const_cast<GreasePencilFrame *>(frame)));
+  return RNA_pointer_create_with_parent(
+      iter->parent,
+      &RNA_GreasePencilFrame,
+      static_cast<void *>(const_cast<GreasePencilFrame *>(frame)));
 }
 
 static int rna_GreasePencilLayer_frames_length(PointerRNA *ptr)
@@ -142,17 +162,16 @@ static int rna_GreasePencilLayer_frames_length(PointerRNA *ptr)
 static bool rna_GreasePencilLayer_frames_lookup_int(PointerRNA *ptr, int index, PointerRNA *r_ptr)
 {
   using namespace blender::bke::greasepencil;
-  GreasePencil &grease_pencil = *rna_grease_pencil(ptr);
   Layer &layer = static_cast<GreasePencilLayer *>(ptr->data)->wrap();
   if (index < 0 || index >= layer.sorted_keys().size()) {
     return false;
   }
   const FramesMapKeyT frame_key = layer.sorted_keys()[index];
   const GreasePencilFrame *frame = layer.frames().lookup_ptr(frame_key);
-
-  r_ptr->owner_id = &grease_pencil.id;
-  r_ptr->type = &RNA_GreasePencilFrame;
-  r_ptr->data = static_cast<void *>(const_cast<GreasePencilFrame *>(frame));
+  rna_pointer_create_with_ancestors(*ptr,
+                                    &RNA_GreasePencilFrame,
+                                    static_cast<void *>(const_cast<GreasePencilFrame *>(frame)),
+                                    *r_ptr);
   return true;
 }
 
@@ -201,8 +220,8 @@ static PointerRNA rna_Frame_drawing_get(PointerRNA *ptr)
   }
 
   const Drawing *drawing = grease_pencil.get_drawing_at(*this_layer, frame_number);
-  return rna_pointer_inherit_refine(
-      ptr, &RNA_GreasePencilDrawing, static_cast<void *>(const_cast<Drawing *>(drawing)));
+  return RNA_pointer_create_with_parent(
+      *ptr, &RNA_GreasePencilDrawing, static_cast<void *>(const_cast<Drawing *>(drawing)));
 }
 
 static void rna_Frame_drawing_set(PointerRNA *frame_ptr,
@@ -304,6 +323,81 @@ static void rna_grease_pencil_active_mask_index_range(
   *max = max_ii(0, BLI_listbase_count(&layer->masks) - 1);
 }
 
+static void tree_node_name_get(blender::bke::greasepencil::TreeNode &node, char *dst)
+{
+  if (!node.name().is_empty()) {
+    strcpy(dst, node.name().c_str());
+  }
+  else {
+    dst[0] = '\0';
+  }
+}
+
+static int tree_node_name_length(blender::bke::greasepencil::TreeNode &node)
+{
+  if (!node.name().is_empty()) {
+    return node.name().size();
+  }
+  return 0;
+}
+
+static std::optional<std::string> tree_node_name_path(blender::bke::greasepencil::TreeNode &node,
+                                                      const char *prefix)
+{
+  using namespace blender::bke::greasepencil;
+  BLI_assert(!node.name().is_empty());
+  const size_t name_length = node.name().size();
+  std::string name_esc(name_length * 2, '\0');
+  BLI_str_escape(name_esc.data(), node.name().c_str(), name_length * 2);
+  return fmt::format("{}[\"{}\"]", prefix, name_esc.c_str());
+}
+
+static StructRNA *rna_GreasePencilTreeNode_refine(PointerRNA *ptr)
+{
+  GreasePencilLayerTreeNode *node = static_cast<GreasePencilLayerTreeNode *>(ptr->data);
+  switch (node->type) {
+    case GP_LAYER_TREE_LEAF:
+      return &RNA_GreasePencilLayer;
+    case GP_LAYER_TREE_GROUP:
+      return &RNA_GreasePencilLayerGroup;
+    default:
+      BLI_assert_unreachable();
+  }
+  return nullptr;
+}
+
+static void rna_GreasePencilTreeNode_name_get(PointerRNA *ptr, char *value)
+{
+  GreasePencilLayerTreeNode *node = static_cast<GreasePencilLayerTreeNode *>(ptr->data);
+  tree_node_name_get(node->wrap(), value);
+}
+
+static int rna_GreasePencilTreeNode_name_length(PointerRNA *ptr)
+{
+  GreasePencilLayerTreeNode *node = static_cast<GreasePencilLayerTreeNode *>(ptr->data);
+  return tree_node_name_length(node->wrap());
+}
+
+static void rna_GreasePencilTreeNode_name_set(PointerRNA *ptr, const char *value)
+{
+  GreasePencil *grease_pencil = rna_grease_pencil(ptr);
+  GreasePencilLayerTreeNode *node = static_cast<GreasePencilLayerTreeNode *>(ptr->data);
+
+  grease_pencil->rename_node(*G_MAIN, node->wrap(), value);
+}
+
+static PointerRNA rna_GreasePencilTreeNode_parent_layer_group_get(PointerRNA *ptr)
+{
+  GreasePencil *grease_pencil = rna_grease_pencil(ptr);
+  GreasePencilLayerTreeNode *node = static_cast<GreasePencilLayerTreeNode *>(ptr->data);
+  /* Return 'None' when node is in the root group. This group is not meant to be seen. */
+  if (node->parent == nullptr || node->parent == grease_pencil->root_group_ptr) {
+    return PointerRNA_NULL;
+  }
+  return RNA_pointer_create_with_parent(
+      *ptr, &RNA_GreasePencilLayerGroup, static_cast<void *>(node->parent));
+}
+
 static void rna_iterator_grease_pencil_layers_begin(CollectionPropertyIterator *iter,
                                                     PointerRNA *ptr)
 {
@@ -342,61 +436,12 @@ static int rna_iterator_grease_pencil_layers_length(PointerRNA *ptr)
   return grease_pencil->layers().size();
 }
 
-static void tree_node_name_get(blender::bke::greasepencil::TreeNode &node, char *dst)
-{
-  if (!node.name().is_empty()) {
-    strcpy(dst, node.name().c_str());
-  }
-  else {
-    dst[0] = '\0';
-  }
-}
-
-static int tree_node_name_length(blender::bke::greasepencil::TreeNode &node)
-{
-  if (!node.name().is_empty()) {
-    return node.name().size();
-  }
-  return 0;
-}
-
-static std::optional<std::string> tree_node_name_path(blender::bke::greasepencil::TreeNode &node,
-                                                      const char *prefix)
-{
-  using namespace blender::bke::greasepencil;
-  BLI_assert(!node.name().is_empty());
-  const size_t name_length = node.name().size();
-  std::string name_esc(name_length * 2, '\0');
-  BLI_str_escape(name_esc.data(), node.name().c_str(), name_length * 2);
-  return fmt::format("{}[\"{}\"]", prefix, name_esc.c_str());
-}
-
 static std::optional<std::string> rna_GreasePencilLayer_path(const PointerRNA *ptr)
 {
   GreasePencilLayer *layer = static_cast<GreasePencilLayer *>(ptr->data);
   return tree_node_name_path(layer->wrap().as_node(), "layers");
 }
 
-static void rna_GreasePencilLayer_name_get(PointerRNA *ptr, char *value)
-{
-  GreasePencilLayer *layer = static_cast<GreasePencilLayer *>(ptr->data);
-  tree_node_name_get(layer->wrap().as_node(), value);
-}
-
-static int rna_GreasePencilLayer_name_length(PointerRNA *ptr)
-{
-  GreasePencilLayer *layer = static_cast<GreasePencilLayer *>(ptr->data);
-  return tree_node_name_length(layer->wrap().as_node());
-}
-
-static void rna_GreasePencilLayer_name_set(PointerRNA *ptr, const char *value)
-{
-  GreasePencil *grease_pencil = rna_grease_pencil(ptr);
-  GreasePencilLayer *layer = static_cast<GreasePencilLayer *>(ptr->data);
-
-  grease_pencil->rename_node(*G_MAIN, layer->wrap().as_node(), value);
-}
-
 static int rna_GreasePencilLayer_pass_index_get(PointerRNA *ptr)
 {
   using namespace blender;
@@ -471,7 +516,7 @@ static void rna_GreasePencilLayer_tint_color_set(PointerRNA *ptr, const float *v
           grease_pencil.attributes_for_write().lookup_or_add_for_write_span<ColorGeometry4f>(
               "tint_color",
               bke::AttrDomain::Layer,
-              bke::AttributeInitVArray(VArray<ColorGeometry4f>::ForSingle(
+              bke::AttributeInitVArray(VArray<ColorGeometry4f>::from_single(
                   ColorGeometry4f(0.0f, 0.0f, 0.0f, 0.0f), grease_pencil.layers().size()))))
   {
     copy_v3_v3(tint_colors.span[layer_idx], values);
@@ -504,7 +549,7 @@ static void rna_GreasePencilLayer_tint_factor_set(PointerRNA *ptr, const float v
           grease_pencil.attributes_for_write().lookup_or_add_for_write_span<ColorGeometry4f>(
               "tint_color",
               bke::AttrDomain::Layer,
-              bke::AttributeInitVArray(VArray<ColorGeometry4f>::ForSingle(
+              bke::AttributeInitVArray(VArray<ColorGeometry4f>::from_single(
                   ColorGeometry4f(0.0f, 0.0f, 0.0f, 0.0f), grease_pencil.layers().size()))))
   {
     tint_colors.span[layer_idx][3] = value;
@@ -538,7 +583,7 @@ static void rna_GreasePencilLayer_radius_offset_set(PointerRNA *ptr, const float
               "radius_offset",
               bke::AttrDomain::Layer,
               bke::AttributeInitVArray(
-                  VArray<float>::ForSingle(0.0f, grease_pencil.layers().size()))))
+                  VArray<float>::from_single(0.0f, grease_pencil.layers().size()))))
   {
     radius_offsets.span[layer_idx] = value;
     radius_offsets.finish();
@@ -559,24 +604,12 @@ static void rna_GreasePencilLayer_matrix_parent_inverse_get(PointerRNA *ptr, flo
   std::copy_n(layer.parent_inverse().base_ptr(), 16, values);
 }
 
-static PointerRNA rna_GreasePencilLayer_parent_layer_group_get(PointerRNA *ptr)
-{
-  blender::bke::greasepencil::Layer &layer = static_cast<GreasePencilLayer *>(ptr->data)->wrap();
-  blender::bke::greasepencil::LayerGroup *layer_group = &layer.parent_group();
-  /* Return None when layer is in the root group. */
-  if (!layer_group->as_node().parent_group()) {
-    return PointerRNA_NULL;
-  }
-  return rna_pointer_inherit_refine(
-      ptr, &RNA_GreasePencilLayerGroup, static_cast<void *>(layer_group));
-}
-
 static PointerRNA rna_GreasePencil_active_layer_get(PointerRNA *ptr)
 {
   GreasePencil *grease_pencil = rna_grease_pencil(ptr);
   if (grease_pencil->has_active_layer()) {
-    return rna_pointer_inherit_refine(
-        ptr, &RNA_GreasePencilLayer, static_cast<void *>(grease_pencil->get_active_layer()));
+    return RNA_pointer_create_with_parent(
+        *ptr, &RNA_GreasePencilLayer, static_cast<void *>(grease_pencil->get_active_layer()));
   }
   return PointerRNA_NULL;
 }
@@ -590,25 +623,12 @@ static void rna_GreasePencil_active_layer_set(PointerRNA *ptr,
   WM_main_add_notifier(NC_GPENCIL | NA_EDITED | NA_SELECTED, grease_pencil);
 }
 
-static PointerRNA rna_GreasePencilLayerGroup_parent_group_get(PointerRNA *ptr)
-{
-  blender::bke::greasepencil::LayerGroup &layer_group =
-      static_cast<GreasePencilLayerTreeGroup *>(ptr->data)->wrap();
-  blender::bke::greasepencil::LayerGroup *parent_group = layer_group.as_node().parent_group();
-  /* Return None when group is in the root group. */
-  if (!parent_group || parent_group == rna_grease_pencil(ptr)->root_group_ptr) {
-    return PointerRNA_NULL;
-  }
-  return rna_pointer_inherit_refine(
-      ptr, &RNA_GreasePencilLayerGroup, static_cast<void *>(parent_group));
-}
-
 static PointerRNA rna_GreasePencil_active_group_get(PointerRNA *ptr)
 {
   GreasePencil *grease_pencil = rna_grease_pencil(ptr);
   if (grease_pencil->has_active_group()) {
-    return rna_pointer_inherit_refine(
-        ptr, &RNA_GreasePencilLayerGroup, static_cast<void *>(grease_pencil->get_active_group()));
+    return RNA_pointer_create_with_parent(
+        *ptr, &RNA_GreasePencilLayerGroup, static_cast<void *>(grease_pencil->get_active_group()));
   }
   return PointerRNA_NULL;
 }
@@ -632,26 +652,6 @@ static std::optional<std::string> rna_GreasePencilLayerGroup_path(const PointerR
   return tree_node_name_path(group->wrap().as_node(), "layer_groups");
 }
 
-static void rna_GreasePencilLayerGroup_name_get(PointerRNA *ptr, char *value)
-{
-  GreasePencilLayerTreeGroup *group = static_cast<GreasePencilLayerTreeGroup *>(ptr->data);
-  tree_node_name_get(group->wrap().as_node(), value);
-}
-
-static int rna_GreasePencilLayerGroup_name_length(PointerRNA *ptr)
-{
-  GreasePencilLayerTreeGroup *group = static_cast<GreasePencilLayerTreeGroup *>(ptr->data);
-  return tree_node_name_length(group->wrap().as_node());
-}
-
-static void rna_GreasePencilLayerGroup_name_set(PointerRNA *ptr, const char *value)
-{
-  GreasePencil *grease_pencil = rna_grease_pencil(ptr);
-  GreasePencilLayerTreeGroup *group = static_cast<GreasePencilLayerTreeGroup *>(ptr->data);
-
-  grease_pencil->rename_node(*G_MAIN, group->wrap().as_node(), value);
-}
-
 static void rna_GreasePencilLayerGroup_is_expanded_set(PointerRNA *ptr, const bool value)
 {
   GreasePencilLayerTreeGroup *group = static_cast<GreasePencilLayerTreeGroup *>(ptr->data);
@@ -908,6 +908,100 @@ static void rna_def_grease_pencil_layer_masks(BlenderRNA *brna, PropertyRNA *cpr
   RNA_def_property_ui_text(prop, "Active Layer Mask Index", "Active index in layer mask array");
 }
 
+static void rna_def_grease_pencil_tree_node(BlenderRNA *brna)
+{
+  StructRNA *srna;
+  PropertyRNA *prop;
+
+  srna = RNA_def_struct(brna, "GreasePencilTreeNode", nullptr);
+  RNA_def_struct_ui_text(
+      srna, "Tree Node", "Grease Pencil node in the layer tree. Either a layer or a group");
+  RNA_def_struct_sdna(srna, "GreasePencilLayerTreeNode");
+  RNA_def_struct_refine_func(srna, "rna_GreasePencilTreeNode_refine");
+
+  /* Name. */
+  prop = RNA_def_property(srna, "name", PROP_STRING, PROP_NONE);
+  RNA_def_property_ui_text(prop, "Name", "The name of the tree node");
+  RNA_def_property_string_funcs(prop,
+                                "rna_GreasePencilTreeNode_name_get",
+                                "rna_GreasePencilTreeNode_name_length",
+                                "rna_GreasePencilTreeNode_name_set");
+  RNA_def_struct_name_property(srna, prop);
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA | NA_RENAME, "rna_grease_pencil_update");
+
+  /* Visibility. */
+  prop = RNA_def_property(srna, "hide", PROP_BOOLEAN, PROP_NONE);
+  RNA_def_property_boolean_sdna(prop, nullptr, "flag", GP_LAYER_TREE_NODE_HIDE);
+  RNA_def_property_ui_icon(prop, ICON_HIDE_OFF, -1);
+  RNA_def_property_ui_text(prop, "Hide", "Set tree node visibility");
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
+
+  /* Lock. */
+  prop = RNA_def_property(srna, "lock", PROP_BOOLEAN, PROP_NONE);
+  RNA_def_property_boolean_sdna(prop, nullptr, "flag", GP_LAYER_TREE_NODE_LOCKED);
+  RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
+  RNA_def_property_ui_text(prop, "Locked", "Protect tree node from editing");
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
+
+  /* Select. */
+  prop = RNA_def_property(srna, "select", PROP_BOOLEAN, PROP_NONE);
+  RNA_def_property_boolean_sdna(prop, nullptr, "flag", GP_LAYER_TREE_NODE_SELECT);
+  RNA_def_property_ui_text(prop, "Select", "Tree node is selected");
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
+
+  /* Onion Skinning. */
+  prop = RNA_def_property(srna, "use_onion_skinning", PROP_BOOLEAN, PROP_NONE);
+  RNA_def_property_ui_icon(prop, ICON_ONIONSKIN_OFF, 1);
+  RNA_def_property_boolean_negative_sdna(
+      prop, nullptr, "flag", GP_LAYER_TREE_NODE_HIDE_ONION_SKINNING);
+  RNA_def_property_ui_text(
+      prop, "Onion Skinning", "Display onion skins before and after the current frame");
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
+
+  /* Use Masks. */
+  prop = RNA_def_property(srna, "use_masks", PROP_BOOLEAN, PROP_NONE);
+  RNA_def_property_ui_icon(prop, ICON_CLIPUV_HLT, -1);
+  RNA_def_property_boolean_negative_sdna(prop, nullptr, "flag", GP_LAYER_TREE_NODE_HIDE_MASKS);
+  RNA_def_property_ui_text(
+      prop,
+      "Use Masks",
+      "The visibility of drawings in this tree node is affected by the layers in the masks list");
+  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
+
+  /* Channel color. */
+  prop = RNA_def_property(srna, "channel_color", PROP_FLOAT, PROP_COLOR);
+  RNA_def_property_float_sdna(prop, nullptr, "color");
+  RNA_def_property_array(prop, 3);
+  RNA_def_property_ui_text(prop, "Channel Color", "Color of the channel in the dope sheet");
+  RNA_def_property_update(prop, NC_GPENCIL | NA_EDITED, nullptr);
+
+  /* Next tree node. */
+  prop = RNA_def_property(srna, "next_node", PROP_POINTER, PROP_NONE);
+  RNA_def_property_pointer_sdna(prop, nullptr, "next");
+  RNA_def_property_struct_type(prop, "GreasePencilTreeNode");
+  RNA_def_property_ui_text(prop, "Next Node", "The layer tree node after (i.e. above) this one");
+  RNA_def_property_clear_flag(prop, PROP_EDITABLE | PROP_ANIMATABLE);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_NO_COMPARISON);
+
+  /* Previous tree node. */
+  prop = RNA_def_property(srna, "prev_node", PROP_POINTER, PROP_NONE);
+  RNA_def_property_pointer_sdna(prop, nullptr, "prev");
+  RNA_def_property_struct_type(prop, "GreasePencilTreeNode");
+  RNA_def_property_ui_text(
+      prop, "Previous Node", "The layer tree node before (i.e. below) this one");
+  RNA_def_property_clear_flag(prop, PROP_EDITABLE | PROP_ANIMATABLE);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_NO_COMPARISON);
+
+  /* Parent group. */
+  prop = RNA_def_property(srna, "parent_group", PROP_POINTER, PROP_NONE);
+  RNA_def_property_struct_type(prop, "GreasePencilLayerGroup");
+  RNA_def_property_pointer_funcs(
+      prop, "rna_GreasePencilTreeNode_parent_layer_group_get", nullptr, nullptr, nullptr);
+  RNA_def_property_ui_text(prop, "Parent Layer Group", "The parent group of this layer tree node");
+  RNA_def_property_clear_flag(prop, PROP_EDITABLE | PROP_ANIMATABLE);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_NO_COMPARISON);
+}
+
 static void rna_def_grease_pencil_layer(BlenderRNA *brna)
 {
   StructRNA *srna;
@@ -924,21 +1018,11 @@ static void rna_def_grease_pencil_layer(BlenderRNA *brna)
       {GP_LAYER_BLEND_DIVIDE, "DIVIDE", 0, "Divide", ""},
       {0, nullptr, 0, nullptr, nullptr}};
 
-  srna = RNA_def_struct(brna, "GreasePencilLayer", nullptr);
+  srna = RNA_def_struct(brna, "GreasePencilLayer", "GreasePencilTreeNode");
   RNA_def_struct_sdna(srna, "GreasePencilLayer");
   RNA_def_struct_ui_text(srna, "Grease Pencil Layer", "Collection of related drawings");
   RNA_def_struct_path_func(srna, "rna_GreasePencilLayer_path");
 
-  /* Name */
-  prop = RNA_def_property(srna, "name", PROP_STRING, PROP_NONE);
-  RNA_def_property_ui_text(prop, "Name", "Layer name");
-  RNA_def_property_string_funcs(prop,
-                                "rna_GreasePencilLayer_name_get",
-                                "rna_GreasePencilLayer_name_length",
-                                "rna_GreasePencilLayer_name_set");
-  RNA_def_struct_name_property(srna, prop);
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA | NA_RENAME, "rna_grease_pencil_update");
-
   /* Frames. */
   prop = RNA_def_property(srna, "frames", PROP_COLLECTION, PROP_NONE);
   RNA_def_property_struct_type(prop, "GreasePencilFrame");
@@ -961,30 +1045,6 @@ static void rna_def_grease_pencil_layer(BlenderRNA *brna)
   RNA_def_property_ui_text(prop, "Masks", "List of Masking Layers");
   rna_def_grease_pencil_layer_masks(brna, prop);
 
-  /* Visibility */
-  prop = RNA_def_property(srna, "hide", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE);
-  RNA_def_property_ui_icon(prop, ICON_HIDE_OFF, -1);
-  RNA_def_property_ui_text(prop, "Hide", "Set layer visibility");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
-  /* Lock */
-  prop = RNA_def_property(srna, "lock", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_LOCKED);
-  RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
-  RNA_def_property_ui_text(
-      prop, "Locked", "Protect layer from further editing and/or frame changes");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
-  /* Select. */
-  prop = RNA_def_property(srna, "select", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_SELECT);
-  RNA_def_property_ui_text(prop, "Select", "Layer is selected for editing in the Dope Sheet");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
   /* Lock Frame. */
   prop = RNA_def_property(srna, "lock_frame", PROP_BOOLEAN, PROP_NONE);
   RNA_def_property_boolean_sdna(
@@ -1000,15 +1060,6 @@ static void rna_def_grease_pencil_layer(BlenderRNA *brna)
   RNA_def_property_ui_text(prop, "Opacity", "Layer Opacity");
   RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
 
-  /* Onion Skinning. */
-  prop = RNA_def_property(srna, "use_onion_skinning", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_ui_icon(prop, ICON_ONIONSKIN_OFF, 1);
-  RNA_def_property_boolean_negative_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE_ONION_SKINNING);
-  RNA_def_property_ui_text(
-      prop, "Onion Skinning", "Display onion skins before and after the current frame");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
   /* Tint Color. */
   prop = RNA_def_property(srna, "tint_color", PROP_FLOAT, PROP_COLOR);
   RNA_def_property_array(prop, 3);
@@ -1040,17 +1091,6 @@ static void rna_def_grease_pencil_layer(BlenderRNA *brna)
   RNA_def_property_ui_text(prop, "Radius Offset", "Radius change to apply to current strokes");
   RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
 
-  /* Use Masks. */
-  prop = RNA_def_property(srna, "use_masks", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_ui_icon(prop, ICON_CLIPUV_HLT, -1);
-  RNA_def_property_boolean_negative_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE_MASKS);
-  RNA_def_property_ui_text(
-      prop,
-      "Use Masks",
-      "The visibility of drawings on this layer is affected by the layers in its masks list");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
   prop = RNA_def_property(srna, "use_lights", PROP_BOOLEAN, PROP_NONE);
   RNA_def_property_boolean_sdna(
       prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_USE_LIGHTS);
@@ -1151,19 +1191,6 @@ static void rna_def_grease_pencil_layer(BlenderRNA *brna)
   RNA_def_property_float_funcs(
       prop, "rna_GreasePencilLayer_matrix_parent_inverse_get", nullptr, nullptr);
 
-  /* Parent layer group. */
-  prop = RNA_def_property(srna, "parent_group", PROP_POINTER, PROP_NONE);
-  RNA_def_property_struct_type(prop, "GreasePencilLayerGroup");
-  RNA_def_property_pointer_funcs(
-      prop, "rna_GreasePencilLayer_parent_layer_group_get", nullptr, nullptr, nullptr);
-  RNA_def_property_ui_text(
-      prop, "Parent Layer Group", "The parent layer group this layer is part of");
-
-  prop = RNA_def_property(srna, "channel_color", PROP_FLOAT, PROP_COLOR);
-  RNA_def_property_float_sdna(prop, "GreasePencilLayerTreeNode", "color");
-  RNA_def_property_array(prop, 3);
-  RNA_def_property_update(prop, NC_GPENCIL | NA_EDITED, nullptr);
-
   RNA_api_grease_pencil_layer(srna);
 }
 
@@ -1209,82 +1236,25 @@ static void rna_def_grease_pencil_layer_group(BlenderRNA *brna)
   StructRNA *srna;
   PropertyRNA *prop;
 
-  srna = RNA_def_struct(brna, "GreasePencilLayerGroup", nullptr);
+  srna = RNA_def_struct(brna, "GreasePencilLayerGroup", "GreasePencilTreeNode");
   RNA_def_struct_sdna(srna, "GreasePencilLayerTreeGroup");
   RNA_def_struct_ui_text(srna, "Grease Pencil Layer Group", "Group of Grease Pencil layers");
   RNA_def_struct_path_func(srna, "rna_GreasePencilLayerGroup_path");
 
-  /* Name */
-  prop = RNA_def_property(srna, "name", PROP_STRING, PROP_NONE);
-  RNA_def_property_ui_text(prop, "Name", "Group name");
-  RNA_def_property_string_funcs(prop,
-                                "rna_GreasePencilLayerGroup_name_get",
-                                "rna_GreasePencilLayerGroup_name_length",
-                                "rna_GreasePencilLayerGroup_name_set");
-  RNA_def_struct_name_property(srna, prop);
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA | NA_RENAME, "rna_grease_pencil_update");
-
-  /* Visibility */
-  prop = RNA_def_property(srna, "hide", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE);
-  RNA_def_property_ui_icon(prop, ICON_HIDE_OFF, -1);
-  RNA_def_property_ui_text(prop, "Hide", "Set layer group visibility");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
-  /* Lock */
-  prop = RNA_def_property(srna, "lock", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_LOCKED);
-  RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
-  RNA_def_property_ui_text(
-      prop, "Locked", "Protect group from further editing and/or frame changes");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
-  /* Use Masks. */
-  prop = RNA_def_property(srna, "use_masks", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_negative_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE_MASKS);
-  RNA_def_property_ui_text(prop,
-                           "Use Masks",
-                           "The visibility of drawings in the layers in this group is affected by "
-                           "the layers in the masks lists");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
-  prop = RNA_def_property(srna, "use_onion_skinning", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_ui_icon(prop, ICON_ONIONSKIN_OFF, 1);
-  RNA_def_property_boolean_negative_sdna(
-      prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_HIDE_ONION_SKINNING);
-  RNA_def_property_ui_text(
-      prop, "Onion Skinning", "Display onion skins before and after the current frame");
-  RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
-
   /* Expanded */
   prop = RNA_def_property(srna, "is_expanded", PROP_BOOLEAN, PROP_NONE);
   RNA_def_property_boolean_sdna(
       prop, "GreasePencilLayerTreeNode", "flag", GP_LAYER_TREE_NODE_EXPANDED);
-  RNA_def_property_ui_text(prop, "Expanded", "The layer groups is expanded in the UI");
+  RNA_def_property_ui_text(prop, "Expanded", "The layer group is expanded in the UI");
   RNA_def_property_flag(prop, PROP_LIB_EXCEPTION);
   RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_boolean_funcs(prop, nullptr, "rna_GreasePencilLayerGroup_is_expanded_set");
   RNA_def_property_update(prop, NC_GPENCIL | ND_DATA, "rna_grease_pencil_update");
 
-  /* Parent group. */
-  prop = RNA_def_property(srna, "parent_group", PROP_POINTER, PROP_NONE);
-  RNA_def_property_struct_type(prop, "GreasePencilLayerGroup");
-  RNA_def_property_pointer_funcs(
-      prop, "rna_GreasePencilLayerGroup_parent_group_get", nullptr, nullptr, nullptr);
-  RNA_def_property_ui_text(prop, "Parent Group", "The parent group this group is part of");
-
   /* Color tag. */
   prop = RNA_def_property(srna, "color_tag", PROP_ENUM, PROP_NONE);
   RNA_def_property_enum_funcs(prop, "rna_group_color_tag_get", "rna_group_color_tag_set", nullptr);
   RNA_def_property_enum_items(prop, enum_layergroup_color_items);
-
-  prop = RNA_def_property(srna, "channel_color", PROP_FLOAT, PROP_COLOR);
-  RNA_def_property_float_sdna(prop, "GreasePencilLayerTreeNode", "color");
-  RNA_def_property_array(prop, 3);
-  RNA_def_property_update(prop, NC_GPENCIL | NA_EDITED, nullptr);
 }
 
 static void rna_def_grease_pencil_layer_groups(BlenderRNA *brna, PropertyRNA *cprop)
@@ -1456,17 +1426,7 @@ static void rna_def_grease_pencil_data(BlenderRNA *brna)
   StructRNA *srna;
   PropertyRNA *prop;
 
-  static const EnumPropertyItem prop_stroke_depth_order_items[] = {
-      {0, "2D", 0, "2D Layers", "Display strokes using Grease Pencil layers to define order"},
-      {GREASE_PENCIL_STROKE_ORDER_3D,
-       "3D",
-       0,
-       "3D Location",
-       "Display strokes using real 3D position in 3D space"},
-      {0, nullptr, 0, nullptr, nullptr},
-  };
-
-  srna = RNA_def_struct(brna, "GreasePencilv3", "ID");
+  srna = RNA_def_struct(brna, "GreasePencil", "ID");
   RNA_def_struct_sdna(srna, "GreasePencil");
   RNA_def_struct_ui_text(srna, "Grease Pencil", "Grease Pencil data-block");
   RNA_def_struct_ui_icon(srna, ICON_OUTLINER_DATA_GREASEPENCIL);
@@ -1534,7 +1494,7 @@ static void rna_def_grease_pencil_data(BlenderRNA *brna)
   /* Uses a single flag, because the depth order can only be 2D or 3D. */
   prop = RNA_def_property(srna, "stroke_depth_order", PROP_ENUM, PROP_NONE);
   RNA_def_property_enum_bitflag_sdna(prop, nullptr, "flag");
-  RNA_def_property_enum_items(prop, prop_stroke_depth_order_items);
+  RNA_def_property_enum_items(prop, rna_enum_stroke_depth_order_items);
   RNA_def_property_ui_text(
       prop,
       "Stroke Depth Order",
@@ -1548,6 +1508,7 @@ static void rna_def_grease_pencil_data(BlenderRNA *brna)
 void RNA_def_grease_pencil(BlenderRNA *brna)
 {
   rna_def_grease_pencil_data(brna);
+  rna_def_grease_pencil_tree_node(brna);
   rna_def_grease_pencil_layer(brna);
   rna_def_grease_pencil_layer_mask(brna);
   rna_def_grease_pencil_layer_group(brna);
