@@ -14,6 +14,8 @@
 #include "ANIM_action.hh"
 #include "ANIM_nla.hh"
 
+#include "BLT_translation.hh"
+
 #include "RNA_define.hh"
 #include "RNA_enum_types.hh"
 
@@ -522,7 +524,7 @@ static void rna_iterator_nlastrip_action_suitable_slots_begin(CollectionProperty
                                                               PointerRNA *ptr)
 {
   NlaStrip *strip = (NlaStrip *)ptr->data;
-  rna_iterator_generic_action_suitable_slots_begin(iter, strip->act);
+  rna_iterator_generic_action_suitable_slots_begin(iter, ptr, strip->act);
 }
 
 static void rna_NlaStrip_action_start_frame_set(PointerRNA *ptr, float value)
@@ -669,7 +671,7 @@ static void rna_NlaStrip_remove(
   }
 
   BKE_nlastrip_remove_and_free(&track->strips, strip, true);
-  RNA_POINTER_INVALIDATE(strip_ptr);
+  strip_ptr->invalidate();
 
   WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_REMOVED, nullptr);
 
@@ -787,6 +789,7 @@ static void rna_def_nlastrip(BlenderRNA *brna)
   RNA_def_property_clear_flag(
       prop, PROP_EDITABLE); /* XXX for now, not editable, since this is dangerous */
   RNA_def_property_enum_items(prop, prop_type_items);
+  RNA_def_property_translation_context(prop, BLT_I18NCONTEXT_ID_ACTION);
   RNA_def_property_ui_text(prop, "Type", "Type of NLA Strip");
   RNA_def_property_update(prop, NC_ANIMATION | ND_NLA | NA_EDITED, "rna_NlaStrip_update");
 
