@@ -21,7 +21,7 @@ static void node_declare(NodeDeclarationBuilder &b)
   b.add_input<decl::String>("Path")
       .subtype(PROP_FILEPATH)
       .path_filter("*.obj")
-      .hide_label()
+      .optional_label()
       .description("Path to a OBJ file");
 
   b.add_output<decl::Geometry>("Instances");
