@@ -57,5 +57,5 @@ void register_node_type_sh_layer_weight()
   ntype.gpu_fn = file_ns::node_shader_gpu_layer_weight;
   ntype.materialx_fn = file_ns::node_shader_materialx;
 
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
